﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ViridiX;
using ViridiX.Linguist;
using ViridiX.Linguist.Network;

namespace ViridiTestUI
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();
		}

		private void btnConnect_Click(object sender, EventArgs e)
		{
			string ipString = txtIPAddress.Text;
			IPAddress address;
			try
			{
				address = IPAddress.Parse(ipString);
			}
			catch (Exception ex)
			{
				MessageBox.Show($"Unable to parse string '{ipString}' as an IP address: {ex.Message}");
				return;
			}

			using (Xbox xbox = new Xbox())
			{
				try
				{
					xbox.Connect(address);
				}
				catch (Exception ex)
				{
					MessageBox.Show($"Unable to connect to xbox at ip '{ipString}': {ex.Message}");
					return;
				}

				if (xbox.FileSystem == null)
				{
					MessageBox.Show($"Unable to list drive letters: Filesystem is null.");
					return;
				}

				
				List<string> driveLetters;
				try
				{
					driveLetters = xbox.FileSystem.DriveLetters;

					if (driveLetters == null)
					{
						throw new IOException("DriveLetters is null.");
					}

				}
				catch (Exception ex)
				{
					MessageBox.Show($"Failed to get drive letters: {ex.Message}");
					return;
				}

				StringBuilder builder = new StringBuilder();
				builder.AppendLine("Drive Letters:");
				for(int driveLetterIndex = 0; driveLetterIndex < driveLetters.Count; driveLetterIndex++)
				{
					string driveLetter = driveLetters[driveLetterIndex];
						
					builder.AppendLine($"Drive[{driveLetterIndex}]: '{driveLetter}'");
				}

				string driveLettersString = builder.ToString();

				MessageBox.Show(driveLettersString);

			}
		}
	}
}
